@@ -6,9 +6,9 @@
 import logging
 import os
 import weakref
-import xobjects as xo
 from abc import ABC, abstractmethod
 from collections import defaultdict
+from importlib.metadata import entry_points
 from pathlib import Path
 from typing import (
     Dict,
@@ -168,11 +168,6 @@ class KernelDict(dict):
     the correct kernel based on the types of the arguments.
     """
 
-    def __getitem__(self, item):
-        if isinstance(item, str):
-            return KernelDispatcher(item, self)
-        return dict.__getitem__(self, item)
-
     def __getattr__(self, attr):
         if attr.startswith("__"):
             raise AttributeError(attr)
@@ -181,8 +176,7 @@ class KernelDict(dict):
 
 class KernelDispatcher:
     """
-    Dispatches a kernel call to the correct kernel based on the types of the
-    arguments.
+    Dispatches a kernel call to the correct kernel.
     """
 
     def __init__(self, kernel_name, kernels):
@@ -194,28 +188,10 @@ class KernelDispatcher:
             raise ValueError(
                 "Kernels can only be called with named arguments."
             )
-
-        classes = []
-        for arg in kwargs.values():
-            if isinstance(arg, xo.HybridClass):
-                overridable = arg._overridable
-                arg_cls = arg._XoStruct
-            elif isinstance(arg, xo.Struct):
-                arg_cls = type(arg)
-                try:
-                    overridable = arg_cls._DressingClass._overridable
-                except AttributeError:
-                    overridable = False
-            else:
-                continue
-
-            if overridable:
-                classes.append(arg_cls)
-
-        return self._kernels[(self._name, tuple(classes))](**kwargs)
+        return self._kernels[self._name](**kwargs)
 
     def set_n_threads(self, n_threads):
-        for (name, _), kernel in self._kernels.items():
+        for name, kernel in self._kernels.items():
             if name == self._name:
                 kernel.description.n_threads = n_threads
 
@@ -230,22 +206,34 @@ class ModuleNotAvailable(object):
 
 class XContext(ABC):
     minimum_alignment = 1
+    allow_prebuilt_kernels = False
 
     def __init__(self):
         self._kernels = KernelDict()
-        self._buffers = []
+        self._buffers = weakref.WeakSet()
+        self._allocations = 0
 
     def __str__(self):
         return type(self).__name__
 
     def new_buffer(self, capacity=1048576):
         buf = self._make_buffer(capacity=capacity)
-        self.buffers.append(weakref.finalize(buf, log.debug, "free buf"))
+        self._buffers.add(buf)
+        self._allocations += 1
         return buf
 
+    def __getstate__(self):
+        state = self.__dict__
+        del state["_buffers"]
+        return state
+
+    def __setstate__(self, state):
+        self.__dict__.update(state)
+        self._buffers = weakref.WeakSet()
+
     @property
     def buffers(self):
-        return self._buffers
+        return list(self._buffers)
 
     @property
     def kernels(self):
@@ -266,6 +254,7 @@ class XContext(ABC):
         extra_classes: Sequence[Type] = (),
         extra_headers: Sequence[SourceType] = (),
         compile: bool = True,  # noqa
+        extra_compile_args: Sequence[str] = (),
     ):
         """
         Adds user-defined kernels to the context. The kernel source
@@ -345,6 +334,7 @@ class XContext(ABC):
             extra_classes=extra_classes,
             extra_headers=extra_headers,
             compile=compile,
+            extra_compile_args=extra_compile_args,
         )
         self.kernels.update(generated_kernels)
 
@@ -360,9 +350,30 @@ class XContext(ABC):
         extra_classes: Sequence[Type],
         extra_headers: Sequence[SourceType],
         compile: bool,
+        extra_compile_args: Sequence[str],
     ) -> Dict[Tuple[str, tuple], KernelType]:
         pass
 
+    def get_installed_c_source_paths(self) -> List[str]:
+        """Returns a list of include paths registered in dependent packages.
+
+        In a package that depends on xobjects, you can register C source paths
+        using the entry point `xobjects.c_sources`. A path to the directory
+        containing the specified module will be added to the include path when
+        building kernels. For example, the following will allow to write
+        ``#include <xtrack/path/to/some/header.h>`` in kernel sources:
+
+         .. code-block:: toml
+            [project.entry-points.xobjects]
+            include = "xtrack"
+        """
+        sources = []
+        for ep in entry_points(group="xobjects", name="include"):
+            module = ep.load()
+            path = Path(module.__file__).parents[1]
+            sources.append(str(path))
+        return sources
+
     @abstractmethod
     def nparray_to_context_array(self, arr):
         return arr
@@ -461,9 +472,13 @@ class XBuffer(ABC):
         self.capacity = newcapacity
 
     def free(self, offset, size):
+        if offset < 0 or offset + size > self.capacity:
+            raise ValueError("Cannot free outside of buffer")
         nch = Chunk(offset, offset + size)
         # insert sorted
-        if offset > self.chunks[-1].start:  # new chuck at the end
+        if len(self.chunks) == 0:
+            self.chunks.append(nch)
+        elif offset > self.chunks[-1].start:  # new chuck at the end
             self.chunks.append(nch)
         else:  # new chuck needs to be inserted
             for ic, ch in enumerate(self.chunks):
@@ -607,14 +622,6 @@ class Kernel:
             classes.append(self.ret.atype)
         return classes
 
-    def get_overridable_classes(self):
-        return [
-            cls
-            for cls in self.get_classes()
-            if hasattr(cls, "_DressingClass")
-            and cls._DressingClass._overridable
-        ]
-
 
 class Source:
     def __init__(self, source, name=None):
