@@ -10,6 +10,7 @@ import sysconfig
 import uuid
 from pathlib import Path
 from typing import Callable, Dict, List, Sequence, Tuple
+import weakref
 
 from .general import _print
 
@@ -17,6 +18,7 @@ import numpy as np
 import scipy as sp
 
 _forbid_compile = False
+_suppress_warnings = False
 
 from .context import (
     Kernel,
@@ -105,6 +107,9 @@ class LinkedArrayCpu(BaseLinkedArray, np.ndarray):
             order="C",
         )
 
+    def copy(self):
+        return np.array(self)
+
 
 def _so_for_module_name(name, containing_dir=".") -> Path:
     # The so file name is something like:
@@ -145,6 +150,8 @@ class ContextCpu(XContext):
         """
         super().__init__()
         self.omp_num_threads = omp_num_threads
+        if omp_num_threads == 0:
+            self.allow_prebuilt_kernels = True
 
     def __str__(self):
         if not self.openmp_enabled:
@@ -162,8 +169,8 @@ class ContextCpu(XContext):
         specialize=True,
         apply_to_source=(),
         save_source_as=None,
-        extra_compile_args: Sequence[str] = ("-O3", "-Wno-unused-function"),
-        extra_link_args: Sequence[str] = ("-O3",),
+        extra_compile_args: Sequence[str] = (),
+        extra_link_args: Sequence[str] = (),
         extra_cdef="",
         extra_classes=(),
         extra_headers=(),
@@ -264,13 +271,16 @@ class ContextCpu(XContext):
         specialize=True,
         apply_to_source=(),
         save_source_as=None,
-        extra_compile_args=("-O3", "-Wno-unused-function"),
-        extra_link_args=("-O3",),
+        extra_compile_args=(),
+        extra_link_args=(),
         extra_cdef="",
         extra_classes=(),
         extra_headers=(),
         compile=True,  # noqa
     ) -> Dict[Tuple[str, tuple], "KernelCpu"]:
+        extra_compile_args += ("-O3", "-Wno-unused-function")
+        extra_link_args += ("-O3",)
+
         # Determine names and paths
         clean_up_so = not module_name
         module_name = module_name or str(uuid.uuid4().hex)
@@ -333,15 +343,14 @@ class ContextCpu(XContext):
             # Only kernel information, but no possibility to call the kernel
             out_kernels = {}
             for pyname, kernel in kernel_descriptions.items():
-                classes = tuple(kernel.description.get_overridable_classes())
-                out_kernels[(pyname, classes)] = KernelCpu(
+                out_kernels[pyname] = KernelCpu(
                     function=None,
                     description=kernel,
                     ffi_interface=None,
                     context=self,
                 )
 
-        for (pyname, _), kernel in out_kernels.items():
+        for pyname, kernel in out_kernels.items():
             kernel.source = source
             kernel.specialized_source = specialized_source
             # TODO: find better implementation?
@@ -367,8 +376,7 @@ class ContextCpu(XContext):
         )
         out_kernels = {}
         for pyname, kernel_desc in kernel_descriptions.items():
-            classes = tuple(kernel_desc.get_overridable_classes())
-            out_kernels[(pyname, classes)] = KernelCpu(
+            out_kernels[pyname] = KernelCpu(
                 function=getattr(module.lib, kernel_desc.c_name),
                 description=kernel_desc,
                 ffi_interface=module.ffi,
@@ -393,7 +401,8 @@ class ContextCpu(XContext):
             _print("Compiling ContextCpu kernels...")
 
         for pyname, kernel in kernel_descriptions.items():
-            if pyname not in cdefs:  # check if kernel not already declared
+            # check if kernel not already declared
+            if f" {kernel.c_name}(" not in cdefs:
                 signature = cdef_from_kernel(kernel, pyname)
                 ffi_interface.cdef(signature)
                 log.debug(f"cffi def {pyname} {signature}")
@@ -403,26 +412,34 @@ class ContextCpu(XContext):
             ffi_interface.cdef("int omp_get_max_threads();")
 
         # Compile
-        xtr_compile_args = ["-std=c99"]
-        xtr_link_args = ["-std=c99"]
+        xtr_compile_args = ["-std=c99", "-DXO_CONTEXT_CPU"]
+        xtr_link_args = ["-std=c99", "-DXO_CONTEXT_CPU"]
         xtr_compile_args += extra_compile_args
         xtr_link_args += extra_link_args
 
         if self.openmp_enabled:
             xtr_compile_args.append("-fopenmp")
             xtr_link_args.append("-fopenmp")
+            xtr_compile_args.append("-DXO_CONTEXT_CPU_OPENMP")
+            xtr_link_args.append("-DXO_CONTEXT_CPU_OPENMP")
+        else:
+            xtr_compile_args.append("-DXO_CONTEXT_CPU_SERIAL")
+            xtr_link_args.append("-DXO_CONTEXT_CPU_SERIAL")
 
-            # https://mac.r-project.org/openmp/
-            # on macos comment the above and uncomment the below flags to compile OpenMP with Xcode clang:
-            # xtr_compile_args.append("-Xclang")
-            # xtr_compile_args.append("-fopenmp")
-            # xtr_link_args.append("-lomp")
+        extra_include_paths = self.get_installed_c_source_paths()
+        include_flags = [f"-I{path}" for path in extra_include_paths]
+        xtr_compile_args.extend(include_flags)
+        xtr_link_args.extend(include_flags)
 
         if os.name == "nt":  # windows
             # TODO: to be handled properly
             xtr_compile_args = []
             xtr_link_args = []
 
+        if _suppress_warnings:
+            xtr_compile_args.append("-w")
+            xtr_link_args.append("-w")
+
         ffi_interface.set_source(
             module_name,
             specialized_source,
@@ -508,6 +525,24 @@ class ContextCpu(XContext):
 
         return module
 
+    @staticmethod
+    def cffi_module_for_c_types(c_types, containing_dir="."):
+        path = Path(containing_dir)
+        for file in path.iterdir():
+            if not file.suffix in [".so", ".dylib", ".dll"]:
+                continue
+            module_name = file.name.split(".")[0]
+            spec = importlib.util.spec_from_file_location(
+                module_name, str(file)
+            )
+            module = importlib.util.module_from_spec(spec)
+
+            typedefs = module.ffi.list_types()[0]
+            if set(c_types) <= set(typedefs):
+                return module_name
+
+        return None
+
     def nparray_to_context_array(self, arr):
         """
         Moves a numpy array to the device memory. No action is performed by
@@ -619,10 +654,12 @@ class ContextCpu(XContext):
     def __getstate__(self):
         state = self.__dict__.copy()
         state["_kernels"] = {}
+        del state["_buffers"]
         return state
 
     def __setstate__(self, state):
         self.__dict__.update(state)
+        self._buffers = weakref.WeakSet()
 
 
 class BufferByteArray(XBuffer):
