@@ -180,6 +180,7 @@ class ContextPyopencl(XContext):
         specialize=True,
         apply_to_source=(),
         save_source_as=None,
+        extra_compile_args=(),
         extra_cdef=None,
         extra_classes=(),
         extra_headers=(),
@@ -218,7 +219,18 @@ class ContextPyopencl(XContext):
             with open(save_source_as, "w") as fid:
                 fid.write(specialized_source)
 
-        prg = cl.Program(self.context, specialized_source).build()
+        extra_include_paths = self.get_installed_c_source_paths()
+        include_flags = [f"-I{path}" for path in extra_include_paths]
+
+        extra_compile_args = (
+            *extra_compile_args,
+            *include_flags,
+            "-cl-std=CL2.0",
+            "-DXO_CONTEXT_CL",
+        )
+        prg = cl.Program(self.context, specialized_source).build(
+            options=extra_compile_args,
+        )
 
         out_kernels = {}
         for pyname, kernel in kernel_descriptions.items():
@@ -234,11 +246,12 @@ class ContextPyopencl(XContext):
             out_kernels[pyname].source = source
             out_kernels[pyname].specialized_source = specialized_source
 
-        kernels_with_classes = {
-            (name, tuple(kernel.description.get_overridable_classes())): kernel
-            for name, kernel in out_kernels.items()
-        }
-        return kernels_with_classes
+        return out_kernels
+
+    def __str__(self):
+        platform_id = cl.get_platforms().index(self.platform)
+        device_id = self.platform.get_devices().index(self.device)
+        return f"{type(self).__name__}:{platform_id}.{device_id}"
 
     def nparray_to_context_array(self, arr):
         """
