@@ -57,9 +57,11 @@ from .typeutils import (
     default_conf,
 )
 
+from .general import Print
 from .scalar import Int64
 from .array import Array
-from .context import Source
+from .context import Source, Arg, Kernel
+from .context_cpu import ContextCpu
 
 log = logging.getLogger(__name__)
 
@@ -471,12 +473,12 @@ class Struct(metaclass=MetaStruct):
         apply_to_source=(),
         save_source_as=None,
         extra_classes=(),
+        extra_compile_args=(),
     ):
         if only_if_needed:
             all_found = True
             for kk, kernel_description in cls._kernels.items():
-                classes = tuple(kernel_description.get_overridable_classes())
-                if (kk, classes) not in context.kernels.keys():
+                if kk not in context.kernels.keys():
                     all_found = False
                     break
             if all_found:
@@ -488,6 +490,7 @@ class Struct(metaclass=MetaStruct):
             extra_classes=[cls] + list(extra_classes),
             apply_to_source=apply_to_source,
             save_source_as=save_source_as,
+            extra_compile_args=extra_compile_args,
         )
 
     def compile_kernels(
@@ -496,6 +499,7 @@ class Struct(metaclass=MetaStruct):
         apply_to_source=(),
         save_source_as=None,
         extra_classes=(),
+        extra_compile_args=(),
     ):
         self.compile_class_kernels(
             context=self._context,
@@ -503,6 +507,7 @@ class Struct(metaclass=MetaStruct):
             apply_to_source=apply_to_source,
             save_source_as=save_source_as,
             extra_classes=extra_classes,
+            extra_compile_args=extra_compile_args,
         )
 
 
